<?php

namespace Drupal\system_tags\Config;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Config\ConfigFactoryOverrideInterface;
use Drupal\Core\Config\StorageInterface;
use Drupal\node\NodeInterface;
use Drupal\system_tags\SystemTagFinder\SystemTagFinderManagerInterface;

/**
 * Class SystemPageConfigOverrider.
 *
 * @package Drupal\system_tags\Config
 */
class SystemPageConfigOverrider implements ConfigFactoryOverrideInterface {

  /**
   * The system tag finder.
   *
   * @var \Drupal\system_tags\SystemTagFinder\SystemTagFinderInterface
   */
  protected $systemTagFinder;

  /**
   * SystemPageConfigOverrider constructor.
   *
   * @param \Drupal\system_tags\SystemTagFinder\SystemTagFinderManagerInterface $system_tag_finder_manager
   *   The system tag finder manager.
   */
  public function __construct(SystemTagFinderManagerInterface $system_tag_finder_manager) {
    $this->systemTagFinder = $system_tag_finder_manager->getInstance(['entity_type' => 'node']);
  }

  /**
   * {@inheritdoc}
   */
  public function loadOverrides($names) {
    $overrides = [];
    if (in_array('system.site', $names, TRUE)) {
      if ($nodes = $this->systemTagFinder->findByTag(SystemTagDefinitions::TAG_ACCESS_DENIED)) {
        $overrides['system.site']['page']['403'] = $this->generatePath(reset($nodes));
      }

      if ($nodes = $this->systemTagFinder->findByTag(SystemTagDefinitions::TAG_HOMEPAGE)) {
        $overrides['system.site']['page']['front'] = $this->generatePath(reset($nodes));
      }

      if ($nodes = $this->systemTagFinder->findByTag(SystemTagDefinitions::TAG_PAGE_NOT_FOUND)) {
        $overrides['system.site']['page']['404'] = $this->generatePath(reset($nodes));
      }
    }

    return $overrides;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheSuffix() {
    return 'SystemTagsOverrider';
  }

  /**
   * {@inheritdoc}
   */
  public function createConfigObject($name, $collection = StorageInterface::DEFAULT_COLLECTION) {
    return NULL;
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheableMetadata($name) {
    return new CacheableMetadata();
  }

  /**
   * Generate a path to the node.
   *
   * @param \Drupal\node\NodeInterface $node
   *   The node.
   *
   * @return string
   *   The path.
   */
  private function generatePath(NodeInterface $node) {
    return sprintf('/%s', $node->toUrl()->getInternalPath());
  }

}
