<?php

namespace Drupal\system_tags;

use Drupal\Core\Entity\EntityFieldManagerInterface;

/**
 * Class SystemTagHelper.
 *
 * @package Drupal\system_tags
 */
class SystemTagHelper implements SystemTagHelperInterface {

  /**
   * The entity field manager.
   *
   * @var \Drupal\Core\Entity\EntityFieldManagerInterface
   */
  protected $entityFieldManager;

  /**
   * SystemTagHelper constructor.
   *
   * @param \Drupal\Core\Entity\EntityFieldManagerInterface $entityFieldManager
   *   The entity field manager.
   */
  public function __construct(EntityFieldManagerInterface $entityFieldManager) {
    $this->entityFieldManager = $entityFieldManager;
  }

  /**
   * {@inheritdoc}
   */
  public function getReferenceFieldName($entityTypeId) {
    $names = &drupal_static(__FUNCTION__);

    if (!isset($names[$entityTypeId])) {
      $names[$entityTypeId] = FALSE;

      $definitions = $this->entityFieldManager->getFieldStorageDefinitions($entityTypeId);
      foreach ($definitions as $definition) {
        if ($definition->getSetting('target_type') === 'system_tag') {
          $names[$entityTypeId] = $definition->getName();
        }
      }
    }

    return $names[$entityTypeId];
  }

}
